/* SaveRange.thor by Troels Walsted Hansen <troels@stud.cs.uit.no>
** $VER: SaveRange.thor 2.1 (6.1.95)
**
** Save a specified range of messages to disk.
**
** History
** 
** v2.00: 
**  This version is only for THOR v2.0 or higher.
**  progressbar
**
** v2.1:
**  Added choice of filename format, as requested by Jostein Trondal and
**    Trond Larsen
*/

options results

/* needs THOR and bbsread.library functions */

p = ' ' || address() || ' ' || show('P',,)
thorport = pos(' THOR.',p)

if thorport > 0 then thorport = word(substr(p,thorport+1),1)
else
do
	say 'No THOR port found!'
	exit 10
end

if ~show('p', 'BBSREAD') then
do
	address command
		"run >nil: `GetEnv THOR/THORPath`bin/LoadBBSRead"
		"WaitForPort BBSREAD"
end

/* get info on messages */

address(bbsread)
GETGLOBALDATA stem GLOBALDATA

address(thorport)
GETGLOBALCONFIG stem GLOBALCONFIG
CURRENTBBS stem CURRENT

if(CURRENT.CONFNAME = "") then
do
	address(bbsread)
        GETCONFLIST '"'CURRENT.BBSNAME'"' CONFLIST
	if(rc ~= 0) then
	do
		address(thorport)
		REQUESTNOTIFY TEXT '"'BBSREAD.LASTERROR'"' BT '"_Ok"'
		exit 5
	end

	address(thorport)
	REQUESTLIST instem CONFLIST title '"Select conf:"' SIZEGADGET
	if(rc ~= 0) then exit
	else CURRENT.CONFNAME = result
end

address(bbsread)
GETCONFDATA bbsname '"'CURRENT.BBSNAME'"' confname '"'CURRENT.CONFNAME'"' stem CONFDATA

if(CONFDATA.FIRSTMSG < 1 | CONFDATA.LASTMSG < 1) then exit

address(thorport)
REQUESTINTEGER MIN '"'CONFDATA.FIRSTMSG'"' MAX '"'CONFDATA.LASTMSG'"' INIT '"'CONFDATA.FIRSTMSG'"' TITLE '"First message in range:"' BT '"_Ok|_Cancel"'
firstmsg = result
if(rc ~= 0 | firstmsg = "RESULT") then exit

REQUESTINTEGER MIN '"'firstmsg'"' MAX '"'CONFDATA.LASTMSG'"' INIT '"'CONFDATA.LASTMSG'"' TITLE '"Last message in range:"' BT '"_Ok|_Cancel"'
lastmsg = result
if(rc ~= 0 | lastmsg = "RESULT") then exit

REQUESTFILE TITLE '"Select directory for messages:"' ID '"'GLOBALCONFIG.SAVEDIR'"' FP PAT '"~"'
if(rc ~= 0 | result = "") then exit
else dir = result

REQUESTNOTIFY TEXT '"Select format for the filenames."' BT '"_Confname.MsgNr|_Subject|Subject._readme"'
if(rc ~= 0) then exit

select
    when(result = 0) then format = 'readme'
    when(result = 1) then format = 'confname'
    when(result = 2) then format = 'subject'
end

/* if user selected a file instead (silly user) reduce it to dirname */

endchar = right(dir,1)

if(endchar ~= ":" & endchar ~= "/") then do
	posi = lastpos("/",dir)
	if(posi = 0) then posi = lastpos(":",dir)
	dir = delstr(dir,posi+1)
end

/* change the illegal filename characters ":" and "/" into "." */

if(pos("/",CURRENT.CONFNAME) ~= 0 | pos(":",CURRENT.CONFNAME) ~= 0) then
	newconfname = translate(CURRENT.CONFNAME,"..","/:")
else newconfname = CURRENT.CONFNAME

/* save the messages with a progressbar */

OPENPROGRESS TITLE '"' || 'Saving messages from ' || CURRENT.CONFNAME || '"' TOTAL lastmsg-firstmsg+1 AT '"_Abort"'
if(rc = 0) then
do
	window = result

	do i=firstmsg to lastmsg
		UPDATEPROGRESS REQ window CURRENT i-firstmsg+1 PT '"' || 'Saving message #' || i || '"'
		if(rc ~= 0) then
		do
			CLOSEPROGRESS REQ window
			exit
		end

		select
			when(format = 'readme') then
			do
                address(bbsread)
				READBRMESSAGE BBSNAME '"'CURRENT.BBSNAME'"' '"'CURRENT.CONFNAME'"' MSGNR '"'i'"' HEADSTEM head
				filename = substr(translate(HEAD.SUBJECT, '___', ' /:'), 1, pos('.', HEAD.SUBJECT)) || 'readme'
				address(thorport)
			end

			when(format = 'confname') then filename = newconfname || '.' || i

			when(format = 'subject') then
			do
                address(bbsread)
				READBRMESSAGE BBSNAME '"'CURRENT.BBSNAME'"' '"'CURRENT.CONFNAME'"' MSGNR '"'i'"' HEADSTEM head
				filename = translate(HEAD.SUBJECT, '___', ' /:')
				address(thorport)
			end
		end

		SAVEMESSAGE BBS '"'CURRENT.BBSNAME'"' CONF '"'CURRENT.CONFNAME'"' MSGNR '"'i'"' FILE '"' || dir || filename || '"'
	end
end
CLOSEPROGRESS REQ window

exit
